#ifndef EIGEN_PCA_H
#define EIGEN_PCA_H 1

#include <iostream>
#include <Eigen/Core>
#include <Eigen/Eigen>
#include <vector>

// Pretty simple and straightforward implementation of Kernel - PCA

using namespace Eigen;

typedef std::pair<double, int> myPair;
typedef std::vector<myPair> PermutationIndices;	


class Kernel
{
// We assume that the original dataset is represented in m x n matrix (m dimension, n points)
// so that points are represented by columns

// in this toy example we materialize the kernel, which is not necessary

public:
	Kernel(MatrixXd & data) 
	{ 
		// kernel is a matrix (n x n)
		_kernel = MatrixXd::Zero(data.cols(), data.cols());

		// an example of kernel
		for (int i=0; i<data.cols();i++)
			for (int j=i; j <data.cols(); j++)
			{
				// an example of kernel projection
				_kernel(i,j) = (data.col(i).dot(data.col(j)));
				_kernel(i,j) *= _kernel(i,j);
			}
	};
	
	const MatrixXd & get() const { return _kernel; };

private:
	MatrixXd _kernel;
};

class PCA
{
public:

	//
	// compute the kernel pca
	//
	void kernel_pca(MatrixXd & dataPoints, unsigned int dimSpace);

	// get
	const MatrixXd & get() const { return _result; };

private:
	MatrixXd _result;
};


#endif
