// eigen_pca_kernel.cpp 
	
#include "eigen_pca.h"

void PCA::kernel_pca(MatrixXd & dataPoints, unsigned int dimSpace)
{
	int m = dataPoints.rows();
	int n = dataPoints.cols();

	// materialize the kernel (this is not really needed)
	Kernel k = Kernel(dataPoints);
	std::cout << "Got the kernel" << std::endl;

	// ''centralize''
	MatrixXd K_Centralized = k.get() - MatrixXd::Ones(n, n)*k.get() - k.get()*MatrixXd::Ones(n, n) + 
		MatrixXd::Ones(n, n)*k.get()*MatrixXd::Ones(n, n); 
	std::cout << "Centralized" << std::endl;

	// compute the eigenvalue on the K_Centralized Matrix
	EigenSolver<MatrixXd> m_solve(K_Centralized);
	std::cout << "got the eigenvalues, eigenvectors";
	VectorXd eigenvalues = m_solve.eigenvalues().real();
	MatrixXd eigenVectors = m_solve.eigenvectors().real();	

	// sort and get the permutation indices
	PermutationIndices pi;
	for (int i = 0 ; i < m; i++)
		pi.push_back(std::make_pair(eigenvalues(i), i));

	sort(pi.begin(), pi.end());

	// get top eigenvectors
	_result = MatrixXd::Zero(m, dimSpace);
	for (unsigned int i = m ; i > m - dimSpace; i--)
		_result.col(m - i) = eigenVectors.col(pi[m-1].second); // permutation indeces

	MatrixXd sqrtE = MatrixXd::Zero(dimSpace, dimSpace);
	for (unsigned int i = 0; i < dimSpace; i++)
		sqrtE(i, i) = sqrt(eigenvalues(i));

	// get the final data projection
	_result = (sqrtE * _result.transpose()).transpose();
}


